@@ -1,13 +1,26 @@
+from __future__ import annotations
+
 import asyncio
+import atexit
+from contextlib import suppress
 from enum import Enum
+import time
+from typing import ClassVar
+import weakref
+
 import kubernetes_asyncio as kubernetes
 
-from distributed.core import rpc
+from distributed.core import Status, rpc
 from distributed.deploy import Cluster
-
-from distributed.utils import Log, Logs, LoopRunner
+from distributed.utils import (
+    Log,
+    Logs,
+    TimeoutError,
+    format_dashboard_link,
+)
 
 from dask_kubernetes.common.auth import ClusterAuth
+from dask_kubernetes.common.utils import namespace_default
 from dask_kubernetes.operator import (
     build_cluster_spec,
     wait_for_service,
@@ -16,6 +29,7 @@ from dask_kubernetes.operator import (
 from dask_kubernetes.common.networking import (
     get_scheduler_address,
     wait_for_scheduler,
+    wait_for_scheduler_comm,
 )
 
 
@@ -103,16 +117,16 @@ class KubeCluster(Cluster):
     KubeCluster.from_name
     """
 
+    _instances: ClassVar[weakref.WeakSet[KubeCluster]] = weakref.WeakSet()
+
     def __init__(
         self,
         name,
-        namespace="default",
+        namespace=None,
         image="ghcr.io/dask/dask:latest",
         n_workers=3,
         resources={},
         env=[],
-        loop=None,
-        asynchronous=False,
         auth=ClusterAuth.DEFAULT,
         port_forward_cluster_ip=None,
         create_mode=CreateMode.CREATE_OR_CONNECT,
@@ -120,8 +134,7 @@ class KubeCluster(Cluster):
         **kwargs,
     ):
         self.name = name
-        # TODO: Set namespace to None and get default namespace from user's context
-        self.namespace = namespace
+        self.namespace = namespace or namespace_default()
         self.image = image
         self.n_workers = n_workers
         self.resources = resources
@@ -130,10 +143,10 @@ class KubeCluster(Cluster):
         self.port_forward_cluster_ip = port_forward_cluster_ip
         self.create_mode = create_mode
         self.shutdown_on_close = shutdown_on_close
-        self._loop_runner = LoopRunner(loop=loop, asynchronous=asynchronous)
-        self.loop = self._loop_runner.loop
 
-        super().__init__(asynchronous=asynchronous, **kwargs)
+        self._instances.add(self)
+
+        super().__init__(**kwargs)
         if not self.asynchronous:
             self._loop_runner.start()
             self.sync(self._start)
@@ -142,6 +155,11 @@ class KubeCluster(Cluster):
     def cluster_name(self):
         return f"{self.name}-cluster"
 
+    @property
+    def dashboard_link(self):
+        host = self.scheduler_address.split("://")[1].split("/")[0].split(":")[0]
+        return format_dashboard_link(host, self.forwarded_dashboard_port)
+
     async def _start(self):
         await ClusterAuth.load_first(self.auth)
         cluster_exists = (await self._get_cluster()) is not None
@@ -190,7 +208,15 @@ class KubeCluster(Cluster):
                 ) from e
             await wait_for_scheduler(cluster_name, self.namespace)
             await wait_for_service(core_api, f"{cluster_name}-service", self.namespace)
-            self.scheduler_comm = rpc(await self._get_scheduler_address())
+            scheduler_address = await self._get_scheduler_address()
+            await wait_for_scheduler_comm(scheduler_address)
+            self.scheduler_comm = rpc(scheduler_address)
+            dashboard_address = await get_scheduler_address(
+                f"{self.name}-cluster-service",
+                self.namespace,
+                port_name="http-dashboard",
+            )
+            self.forwarded_dashboard_port = dashboard_address.split(":")[-1]
 
     async def _connect_cluster(self):
         if self.shutdown_on_close is None:
@@ -209,7 +235,15 @@ class KubeCluster(Cluster):
             service_name = f'{cluster_spec["metadata"]["name"]}-service'
             await wait_for_scheduler(self.cluster_name, self.namespace)
             await wait_for_service(core_api, service_name, self.namespace)
-            self.scheduler_comm = rpc(await self._get_scheduler_address())
+            scheduler_address = await self._get_scheduler_address()
+            await wait_for_scheduler_comm(scheduler_address)
+            self.scheduler_comm = rpc(scheduler_address)
+            dashboard_address = await get_scheduler_address(
+                service_name,
+                self.namespace,
+                port_name="http-dashboard",
+            )
+            self.forwarded_dashboard_port = dashboard_address.split(":")[-1]
 
     async def _get_cluster(self):
         async with kubernetes.client.api_client.ApiClient() as api_client:
@@ -363,11 +397,11 @@ class KubeCluster(Cluster):
                 name=f"{self.name}-cluster-{name}",
             )
 
-    def close(self):
+    def close(self, timeout=3600):
         """Delete the dask cluster"""
-        return self.sync(self._close)
+        return self.sync(self._close, timeout=timeout)
 
-    async def _close(self):
+    async def _close(self, timeout=None):
         await super()._close()
         if self.shutdown_on_close:
             async with kubernetes.client.api_client.ApiClient() as api_client:
@@ -379,7 +413,12 @@ class KubeCluster(Cluster):
                     namespace=self.namespace,
                     name=self.cluster_name,
                 )
+            start = time.time()
             while (await self._get_cluster()) is not None:
+                if time.time() > start + timeout:
+                    raise TimeoutError(
+                        f"Timed out deleting cluster resource {self.cluster_name}"
+                    )
                 await asyncio.sleep(1)
 
     def scale(self, n, worker_group="default"):
@@ -412,7 +451,7 @@ class KubeCluster(Cluster):
                 plural="daskworkergroups",
                 namespace=self.namespace,
                 name=f"{self.name}-cluster-{worker_group}-worker-group",
-                body={"spec": {"worker": {"replicas": n}}},
+                body={"spec": {"replicas": n}},
             )
 
     def adapt(self, *args, **kwargs):
@@ -436,30 +475,28 @@ class KubeCluster(Cluster):
                     {
                         "name": "scheduler",
                         "image": self.image,
-                        "args": [
-                            "dask-scheduler",
-                        ],
+                        "args": ["dask-scheduler", "--host", "0.0.0.0"],
                         "env": env,
                         "resources": self.resources,
                         "ports": [
                             {
-                                "name": "comm",
+                                "name": "tcp-comm",
                                 "containerPort": 8786,
                                 "protocol": "TCP",
                             },
                             {
-                                "name": "dashboard",
+                                "name": "http-dashboard",
                                 "containerPort": 8787,
                                 "protocol": "TCP",
                             },
                         ],
                         "readinessProbe": {
-                            "tcpSocket": {"port": "comm"},
+                            "httpGet": {"port": "http-dashboard", "path": "/health"},
                             "initialDelaySeconds": 5,
                             "periodSeconds": 10,
                         },
                         "livenessProbe": {
-                            "tcpSocket": {"port": "comm"},
+                            "httpGet": {"port": "http-dashboard", "path": "/health"},
                             "initialDelaySeconds": 15,
                             "periodSeconds": 20,
                         },
@@ -474,16 +511,16 @@ class KubeCluster(Cluster):
                 },
                 "ports": [
                     {
-                        "name": "comm",
+                        "name": "tcp-comm",
                         "protocol": "TCP",
                         "port": 8786,
-                        "targetPort": "comm",
+                        "targetPort": "tcp-comm",
                     },
                     {
-                        "name": "dashboard",
+                        "name": "http-dashboard",
                         "protocol": "TCP",
                         "port": 8787,
-                        "targetPort": "dashboard",
+                        "targetPort": "http-dashboard",
                     },
                 ],
             },
@@ -506,7 +543,8 @@ class KubeCluster(Cluster):
                         "image": self.image,
                         "args": [
                             "dask-worker",
-                            f"tcp://{service_name}.{self.namespace}.svc.cluster.local:8786",
+                            "--name",
+                            "$(DASK_WORKER_NAME)",
                         ],
                         "env": env,
                         "resources": self.resources,
@@ -537,3 +575,19 @@ class KubeCluster(Cluster):
         >>> cluster = KubeCluster.from_name(name="simple-cluster")
         """
         return cls(name=name, create_mode=CreateMode.CONNECT_ONLY, **kwargs)
+
+
+@atexit.register
+def reap_clusters():
+    async def _reap_clusters():
+        for cluster in list(KubeCluster._instances):
+            if cluster.shutdown_on_close and cluster.status != Status.closed:
+                await ClusterAuth.load_first(cluster.auth)
+                with suppress(TimeoutError):
+                    if cluster.asynchronous:
+                        await cluster.close(timeout=10)
+                    else:
+                        cluster.close(timeout=10)
+
+    loop = asyncio.get_event_loop()
+    loop.run_until_complete(_reap_clusters())
